//
// Copyright 2022 Vulcanize, Inc.
//
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { createLibp2p } from '@cerc-io/libp2p';
import wrtc from 'wrtc';
import debug from 'debug';
import { noise } from '@chainsafe/libp2p-noise';
import { mplex } from '@libp2p/mplex';
import { WebRTCDirectNodeType, webRTCDirect } from '@cerc-io/webrtc-direct';
import { floodsub } from '@libp2p/floodsub';
import { pubsubPeerDiscovery } from '@libp2p/pubsub-peer-discovery';
import { multiaddr } from '@multiformats/multiaddr';
import { createFromJSON } from '@libp2p/peer-id-factory';
import { HOP_TIMEOUT, PUBSUB_DISCOVERY_INTERVAL, PUBSUB_SIGNATURE_POLICY, WEBRTC_PORT_RANGE } from './constants.js';
import { PeerHearbeatChecker } from './peer-heartbeat-checker.js';
import { dialWithRetry } from './utils/index.js';
var log = debug('laconic:relay');
export function createRelayNode(init) {
    return __awaiter(this, void 0, void 0, function () {
        var listenMultiaddrs, announceMultiaddrs, peerId, node, peerHeartbeatChecker;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    listenMultiaddrs = ["/ip4/".concat(init.host, "/tcp/").concat(init.port, "/http/p2p-webrtc-direct")];
                    announceMultiaddrs = [];
                    if (init.announceDomain) {
                        announceMultiaddrs.push("/dns4/".concat(init.announceDomain, "/tcp/443/https/p2p-webrtc-direct"));
                    }
                    if (!init.peerIdObj) return [3 /*break*/, 2];
                    return [4 /*yield*/, createFromJSON(init.peerIdObj)];
                case 1:
                    peerId = _a.sent();
                    _a.label = 2;
                case 2: return [4 /*yield*/, createLibp2p({
                        peerId: peerId,
                        addresses: {
                            listen: listenMultiaddrs,
                            announce: announceMultiaddrs
                        },
                        transports: [
                            webRTCDirect({
                                wrtc: wrtc,
                                enableSignalling: true,
                                nodeType: WebRTCDirectNodeType.Relay,
                                initiatorOptions: { webRTCPortRange: WEBRTC_PORT_RANGE },
                                receiverOptions: { webRTCPortRange: WEBRTC_PORT_RANGE }
                            })
                        ],
                        connectionEncryption: [noise()],
                        streamMuxers: [mplex()],
                        pubsub: floodsub({ globalSignaturePolicy: PUBSUB_SIGNATURE_POLICY }),
                        peerDiscovery: [
                            pubsubPeerDiscovery({
                                interval: PUBSUB_DISCOVERY_INTERVAL
                            })
                        ],
                        relay: {
                            enabled: true,
                            hop: {
                                enabled: true,
                                timeout: HOP_TIMEOUT
                            },
                            advertise: {
                                enabled: true
                            }
                        },
                        connectionManager: {
                            autoDial: false
                        }
                    })];
                case 3:
                    node = _a.sent();
                    peerHeartbeatChecker = new PeerHearbeatChecker(node, init.pingInterval);
                    console.log("Relay node started with id ".concat(node.peerId.toString()));
                    console.log('Listening on:');
                    node.getMultiaddrs().forEach(function (ma) { return console.log(ma.toString()); });
                    // Listen for peers connection
                    node.addEventListener('peer:connect', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                        var connection;
                        var _this = this;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    connection = evt.detail;
                                    log("Connected to ".concat(connection.remotePeer.toString(), " using multiaddr ").concat(connection.remoteAddr.toString()));
                                    // Start heartbeat check for peer
                                    return [4 /*yield*/, peerHeartbeatChecker.start(connection.remotePeer, function () { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                                            return [2 /*return*/, _handleDeadConnections(node, connection.remotePeer)];
                                        }); }); })];
                                case 1:
                                    // Start heartbeat check for peer
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                    // Listen for peers disconnecting
                    // peer:disconnect event is trigerred when all connections to a peer close
                    // https://github.com/libp2p/js-libp2p-interfaces/blob/master/packages/interface-libp2p/src/index.ts#L64
                    node.addEventListener('peer:disconnect', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                        var connection, remoteAddr;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    log('event peer:disconnect', evt);
                                    connection = evt.detail;
                                    remoteAddr = connection.remoteAddr;
                                    log("Disconnected from ".concat(connection.remotePeer.toString(), " using multiaddr ").concat(remoteAddr.toString()));
                                    // Stop connection check for disconnected peer
                                    peerHeartbeatChecker.stop(connection.remotePeer);
                                    if (!init.relayPeers.includes(remoteAddr.toString())) return [3 /*break*/, 2];
                                    return [4 /*yield*/, dialWithRetry(node, remoteAddr, {
                                            redialInterval: init.redialInterval,
                                            maxRetry: init.maxDialRetry
                                        }).catch(function (error) { return console.log(error.message); })];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2: return [2 /*return*/];
                            }
                        });
                    }); });
                    if (!init.relayPeers.length) return [3 /*break*/, 5];
                    console.log('Dialling relay peers');
                    return [4 /*yield*/, _dialRelayPeers(node, init.relayPeers, init.maxDialRetry, init.redialInterval)];
                case 4:
                    _a.sent();
                    _a.label = 5;
                case 5: return [2 /*return*/, node];
            }
        });
    });
}
function _dialRelayPeers(node, relayPeersList, maxDialRetry, redialInterval) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        return __generator(this, function (_a) {
            relayPeersList.forEach(function (relayPeer) { return __awaiter(_this, void 0, void 0, function () {
                var relayMultiaddr;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            relayMultiaddr = multiaddr(relayPeer);
                            return [4 /*yield*/, dialWithRetry(node, relayMultiaddr, {
                                    redialInterval: redialInterval,
                                    maxRetry: maxDialRetry
                                }).catch(function (error) { return console.log(error.message); })];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
function _handleDeadConnections(node, remotePeerId) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    // Close existing connections of remote peer
                    log("Closing connections for ".concat(remotePeerId));
                    return [4 /*yield*/, node.hangUp(remotePeerId)];
                case 1:
                    _a.sent();
                    log('Closed');
                    return [2 /*return*/];
            }
        });
    });
}
//# sourceMappingURL=relay.js.map